'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _merge2 = require('lodash/merge');

var _merge3 = _interopRequireDefault(_merge2);

var _pick2 = require('lodash/pick');

var _pick3 = _interopRequireDefault(_pick2);

var _isUndefined2 = require('lodash/isUndefined');

var _isUndefined3 = _interopRequireDefault(_isUndefined2);

var _isEqual2 = require('lodash/isEqual');

var _isEqual3 = _interopRequireDefault(_isEqual2);

var _debounce2 = require('lodash/debounce');

var _debounce3 = _interopRequireDefault(_debounce2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _isFunction2 = require('lodash/isFunction');

var _isFunction3 = _interopRequireDefault(_isFunction2);

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var React = _interopRequireWildcard(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _elementResizeEvent = require('element-resize-event');

var _elementResizeEvent2 = _interopRequireDefault(_elementResizeEvent);

var _domLib = require('dom-lib');

var _Row = require('./Row');

var _Row2 = _interopRequireDefault(_Row);

var _CellGroup = require('./CellGroup');

var _CellGroup2 = _interopRequireDefault(_CellGroup);

var _Scrollbar = require('./Scrollbar');

var _Scrollbar2 = _interopRequireDefault(_Scrollbar);

var _utils = require('./utils');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var handleClass = { add: _domLib.addClass, remove: _domLib.removeClass };
var ReactChildren = React.Children;
var CELL_PADDING_HEIGHT = 26;

function findRowKeys(rows, rowKey, expanded) {
  var keys = [];
  rows.forEach(function (item) {
    if (item.children) {
      keys.push(item[rowKey]);
      keys = [].concat(_toConsumableArray(keys), _toConsumableArray(findRowKeys(item.children, rowKey)));
    } else if (expanded) {
      keys.push(item[rowKey]);
    }
  });
  return keys;
}

var Table = function (_React$Component) {
  _inherits(Table, _React$Component);

  function Table(props) {
    _classCallCheck(this, Table);

    var _this = _possibleConstructorReturn(this, (Table.__proto__ || Object.getPrototypeOf(Table)).call(this, props));

    _initialiseProps.call(_this);

    var width = props.width,
        data = props.data,
        rowKey = props.rowKey,
        defaultExpandAllRows = props.defaultExpandAllRows,
        renderRowExpanded = props.renderRowExpanded,
        defaultExpandedRowKeys = props.defaultExpandedRowKeys;

    var expandedRowKeys = defaultExpandAllRows ? findRowKeys(data, rowKey, (0, _isFunction3.default)(renderRowExpanded)) : defaultExpandedRowKeys || [];

    _this.state = {
      expandedRowKeys: expandedRowKeys,
      width: width || 0,
      columnWidth: 0,
      dataKey: 0,
      shouldFixedColumn: false,
      contentHeight: 0,
      contentWidth: 0,
      tableRowsMaxHeight: []
    };
    return _this;
  }

  _createClass(Table, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      var _this2 = this;

      var _props = this.props,
          _props$children = _props.children,
          children = _props$children === undefined ? [] : _props$children,
          isTree = _props.isTree,
          rowKey = _props.rowKey;

      var shouldFixedColumn = Array.from(children).some(function (child) {
        return (0, _get3.default)(child, 'props.fixed');
      });

      if (isTree && !rowKey) {
        throw new Error('The `rowKey` is required when set isTree');
      }

      this.scrollY = 0;
      this.scrollX = 0;
      this.wheelHandler = new _domLib.WheelHandler(function (deltaX, deltaY) {
        _this2.handleWheel(deltaX, deltaY);
        _this2.scrollbarX.onWheelScroll(deltaX);
        _this2.scrollbarY.onWheelScroll(deltaY);
      }, this.shouldHandleWheelX, this.shouldHandleWheelY);
      this.setState({ shouldFixedColumn: shouldFixedColumn });
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.calculateTableWidth();
      this.calculateTableContextHeight();
      this.calculateRowMaxHeight();
      (0, _elementResizeEvent2.default)(this.table, (0, _debounce3.default)(this.calculateTableWidth, 400));
    }
  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps, nextState) {
      return !(0, _isEqual3.default)(this.props, nextProps) || !(0, _isEqual3.default)(this.state, nextState);
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      this.calculateTableContextHeight();
      this.calculateTableContentWidth();
      this.calculateRowMaxHeight();
      this.updatePosition();
    }
  }, {
    key: 'getExpandedRowKeys',
    value: function getExpandedRowKeys() {
      var expandedRowKeys = this.props.expandedRowKeys;

      return (0, _isUndefined3.default)(expandedRowKeys) ? this.state.expandedRowKeys : expandedRowKeys;
    }
  }, {
    key: 'getScrollCellGroups',
    value: function getScrollCellGroups() {
      return this.table.querySelectorAll('.' + this.addPrefix('cell-group-scroll'));
    }
  }, {
    key: 'getFixedCellGroups',
    value: function getFixedCellGroups() {
      return this.table.querySelectorAll('.' + this.addPrefix('cell-group-fixed'));
    }
  }, {
    key: 'getCells',
    value: function getCells() {
      var _this3 = this;

      var left = 0; // Cell left margin
      var headerCells = []; // Table header cell
      var bodyCells = []; // Table body cell
      var columns = this.props.children;

      if (!columns) {
        return {
          headerCells: headerCells,
          bodyCells: bodyCells,
          allColumnsWidth: left
        };
      }

      var tableWidth = this.state.width;
      var _props2 = this.props,
          sortColumn = _props2.sortColumn,
          sortType = _props2.sortType,
          onSortColumn = _props2.onSortColumn,
          rowHeight = _props2.rowHeight,
          headerHeight = _props2.headerHeight;

      var _getTotalByColumns = (0, _utils.getTotalByColumns)(columns),
          totalFlexGrow = _getTotalByColumns.totalFlexGrow,
          totalWidth = _getTotalByColumns.totalWidth;

      ReactChildren.forEach(columns, function (column, index) {
        if (React.isValidElement(column)) {
          var columnChildren = column.props.children;
          var _column$props = column.props,
              _width = _column$props.width,
              resizable = _column$props.resizable,
              flexGrow = _column$props.flexGrow,
              minWidth = _column$props.minWidth;


          if (resizable && flexGrow) {
            console.warn('Cannot set \'resizable\' and \'flexGrow\' together in <Column>, column index: ' + index);
          }

          if (columnChildren.length !== 2) {
            throw new Error('Component <HeaderCell> and <Cell> is required, column index: ' + index + ' ');
          }

          var nextWidth = _this3.state[columnChildren[1].props.dataKey + '_' + index + '_width'] || _width || 0;

          if (tableWidth && flexGrow && totalFlexGrow) {
            nextWidth = Math.max((tableWidth - totalWidth) / totalFlexGrow * flexGrow, minWidth || 60);
          }

          var columnHandledProps = ['align', 'width', 'fixed', 'resizable', 'flexGrow', 'minWidth', 'colSpan'];

          var cellProps = _extends({}, (0, _pick3.default)(column.props, columnHandledProps), {
            left: left,
            index: index,
            headerHeight: headerHeight,
            key: index,
            width: nextWidth,
            height: rowHeight,
            firstColumn: index === 0,
            lastColumn: index === columns.length - 1
          });

          var headerCellsProps = {
            dataKey: columnChildren[1].props.dataKey,
            isHeaderCell: true,
            sortable: column.props.sortable,
            sortColumn: sortColumn,
            sortType: sortType,
            onSortColumn: onSortColumn,
            flexGrow: flexGrow
          };

          if (resizable) {
            (0, _merge3.default)(headerCellsProps, {
              onColumnResizeEnd: _this3.handleColumnResizeEnd,
              onColumnResizeStart: _this3.handleColumnResizeStart,
              onColumnResizeMove: _this3.handleColumnResizeMove
            });
          }

          headerCells.push(React.cloneElement(columnChildren[0], _extends({}, cellProps, headerCellsProps)));
          bodyCells.push(React.cloneElement(columnChildren[1], cellProps));

          left += nextWidth;
        }
      });

      return {
        headerCells: headerCells,
        bodyCells: bodyCells,
        allColumnsWidth: left
      };
    }

    // 处理移动端 Touch 事件,  Start 的时候初始化 x,y


    // 处理移动端 Touch 事件, Move 的时候初始化，更新 scroll

  }, {
    key: 'updatePosition',
    value: function updatePosition() {
      /**
       * 当存在锁定列情况处理
       */
      if (this.state.shouldFixedColumn) {
        this.updatePositionByFixedCell();
      } else {
        var wheelStyle = {};
        var headerStyle = {};
        (0, _domLib.translateDOMPositionXY)(wheelStyle, this.scrollX, this.scrollY);
        (0, _domLib.translateDOMPositionXY)(headerStyle, this.scrollX, 0);

        this.wheelWrapper && (0, _domLib.addStyle)(this.wheelWrapper, wheelStyle);
        this.headerWrapper && (0, _domLib.addStyle)(this.headerWrapper, headerStyle);
      }
      handleClass[this.scrollY < 0 ? 'add' : 'remove'](this.tableHeader, this.addPrefix('cell-group-shadow'));
    }
  }, {
    key: 'updatePositionByFixedCell',
    value: function updatePositionByFixedCell() {
      var _this4 = this;

      var wheelGroupStyle = {};
      var wheelStyle = {};
      var scrollGroups = this.getScrollCellGroups();
      var fixedGroups = this.getFixedCellGroups();

      (0, _domLib.translateDOMPositionXY)(wheelGroupStyle, this.scrollX, 0);
      (0, _domLib.translateDOMPositionXY)(wheelStyle, 0, this.scrollY);

      Array.from(scrollGroups).forEach(function (group) {
        (0, _domLib.addStyle)(group, wheelGroupStyle);
      });
      if (this.wheelWrapper) {
        (0, _domLib.addStyle)(this.wheelWrapper, wheelStyle);
      }

      Array.from(fixedGroups).forEach(function (group) {
        handleClass[_this4.scrollX < 0 ? 'add' : 'remove'](group, _this4.addPrefix('cell-group-shadow'));
      });
    }
  }, {
    key: 'calculateRowMaxHeight',
    value: function calculateRowMaxHeight() {
      var _this5 = this;

      var wordWrap = this.props.wordWrap;

      if (wordWrap) {
        var _tableRowsMaxHeight = [];
        this.tableRows.forEach(function (row) {
          var cells = row.querySelectorAll('.' + _this5.addPrefix('cell-wrap')) || [];
          var maxHeight = 0;
          Array.from(cells).forEach(function (cell) {
            var h = (0, _domLib.getHeight)(cell);
            maxHeight = Math.max(maxHeight, h);
          });
          _tableRowsMaxHeight.push(maxHeight);
        });
        this.setState({ tableRowsMaxHeight: _tableRowsMaxHeight });
      }
    }
  }, {
    key: 'calculateTableContentWidth',
    value: function calculateTableContentWidth() {
      var table = this.table;
      var row = table.querySelectorAll('.' + this.addPrefix('row-header'))[0];
      var contentWidth = (0, _domLib.getWidth)(row);

      this.setState({ contentWidth: contentWidth });
      // 这里 -10 是为了让滚动条不挡住内容部分
      this.minScrollX = -(contentWidth - this.state.width) - 10;
      if (this.state.contentWidth !== contentWidth) {
        this.scrollX = 0;
        this.scrollbarX && this.scrollbarX.resetScrollBarPosition();
      }
    }
  }, {
    key: 'calculateTableContextHeight',
    value: function calculateTableContextHeight() {
      var table = this.table;
      var rows = table.querySelectorAll('.' + this.addPrefix('row'));
      var _props3 = this.props,
          height = _props3.height,
          rowHeight = _props3.rowHeight,
          headerHeight = _props3.headerHeight;

      var contentHeight = 0;
      Array.from(rows).forEach(function (row) {
        contentHeight += (0, _domLib.getHeight)(row);
      });

      var nextContentHeight = contentHeight - (headerHeight || rowHeight);
      this.setState({
        contentHeight: nextContentHeight
      });
      // 这里 -10 是为了让滚动条不挡住内容部分
      this.minScrollY = -(contentHeight - height) - 10;
      if (this.state.contentHeight !== nextContentHeight) {
        // this.scrollY = 0;
        // this.scrollbarY && this.scrollbarY.resetScrollBarPosition(30);
      }
    }
  }, {
    key: 'shouldRenderExpandedRow',
    value: function shouldRenderExpandedRow(rowData) {
      var _props4 = this.props,
          rowKey = _props4.rowKey,
          renderRowExpanded = _props4.renderRowExpanded,
          isTree = _props4.isTree;

      var expandedRowKeys = this.getExpandedRowKeys() || [];
      return expandedRowKeys.some(function (key) {
        return key === rowData[rowKey];
      }) && (0, _isFunction3.default)(renderRowExpanded) && !isTree;
    }
  }, {
    key: 'renderRowData',
    value: function renderRowData(bodyCells, rowData, props, shouldRenderExpandedRow) {
      var _this6 = this;

      var _props5 = this.props,
          onRowClick = _props5.onRowClick,
          renderTreeToggle = _props5.renderTreeToggle,
          rowKey = _props5.rowKey,
          wordWrap = _props5.wordWrap;


      var hasChildren = this.props.isTree && rowData.children && Array.isArray(rowData.children);
      var nextRowKey = rowData[rowKey] || '_' + (Math.random() * 1e18).toString(36).slice(0, 5).toUpperCase() + '_' + props.index;

      var rowProps = {
        rowRef: function rowRef(ref) {
          _this6.tableRows[props.index] = ref;
        },
        key: props.index,
        width: props.rowWidth,
        height: props.rowHeight,
        top: props.top,
        onClick: function onClick() {
          onRowClick && onRowClick(rowData);
        }
      };

      var cells = bodyCells.map(function (cell) {
        return React.cloneElement(cell, {
          hasChildren: hasChildren,
          layer: props.layer,
          height: props.rowHeight,
          rowIndex: props.index,
          renderTreeToggle: renderTreeToggle,
          onTreeToggle: _this6.handleTreeToggle,
          rowKey: nextRowKey,
          rowData: rowData,
          wordWrap: wordWrap
        });
      });

      var row = this.renderRow(rowProps, cells, shouldRenderExpandedRow, rowData);

      // insert children
      if (hasChildren) {
        props.layer += 1;

        var _expandedRowKeys = this.getExpandedRowKeys() || [];
        var open = _expandedRowKeys.some(function (key) {
          return key === rowData[rowKey];
        });

        var childrenClasses = (0, _classnames2.default)(this.addPrefix('row-has-children'), _defineProperty({}, this.addPrefix('row-open'), open));

        return React.createElement(
          'div',
          { className: childrenClasses, key: props.index, 'data-layer': props.layer },
          row,
          React.createElement(
            'div',
            { className: this.addPrefix('row-children') },
            rowData.children.map(function (child, index) {
              return _this6.renderRowData(bodyCells, child, _extends({}, props, {
                index: index
              }));
            })
          )
        );
      }

      return row;
    }
  }, {
    key: 'renderRow',
    value: function renderRow(props, cells, shouldRenderExpandedRow, rowData) {
      var shouldFixedColumn = this.state.shouldFixedColumn;
      // IF there are fixed columns, add a fixed group

      if (shouldFixedColumn) {
        var fixedCells = cells.filter(function (cell) {
          return cell.props.fixed;
        });
        var otherCells = cells.filter(function (cell) {
          return !cell.props.fixed;
        });
        var fixedCellGroupWidth = 0;

        for (var i = 0; i < fixedCells.length; i += 1) {
          fixedCellGroupWidth += fixedCells[i].props.width;
        }

        return React.createElement(
          _Row2.default,
          props,
          React.createElement(
            _CellGroup2.default,
            {
              fixed: true,
              height: props.isHeaderRow ? props.headerHeight : props.height,
              width: fixedCellGroupWidth
            },
            (0, _utils.colSpanCells)(fixedCells)
          ),
          React.createElement(
            _CellGroup2.default,
            null,
            (0, _utils.colSpanCells)(otherCells)
          ),
          shouldRenderExpandedRow && this.renderRowExpanded(rowData)
        );
      }

      return React.createElement(
        _Row2.default,
        props,
        React.createElement(
          _CellGroup2.default,
          null,
          (0, _utils.colSpanCells)(cells)
        ),
        shouldRenderExpandedRow && this.renderRowExpanded(rowData)
      );
    }
  }, {
    key: 'renderRowExpanded',
    value: function renderRowExpanded(rowData) {
      var _props6 = this.props,
          renderRowExpanded = _props6.renderRowExpanded,
          rowExpandedHeight = _props6.rowExpandedHeight;

      var styles = {
        height: rowExpandedHeight
      };

      if (typeof renderRowExpanded === 'function') {
        return React.createElement(
          'div',
          { className: this.addPrefix('row-expanded'), style: styles },
          renderRowExpanded(rowData)
        );
      }

      return null;
    }
  }, {
    key: 'renderMouseArea',
    value: function renderMouseArea() {
      var _this7 = this;

      var _props7 = this.props,
          height = _props7.height,
          headerHeight = _props7.headerHeight;

      var styles = { height: height };

      return React.createElement(
        'div',
        {
          ref: function ref(_ref) {
            _this7.mouseArea = _ref;
          },
          className: this.addPrefix('mouse-area'),
          style: styles
        },
        React.createElement('span', {
          style: {
            height: headerHeight - 1
          }
        })
      );
    }
  }, {
    key: 'renderTableHeader',
    value: function renderTableHeader(headerCells, rowWidth) {
      var _this8 = this;

      var _props8 = this.props,
          rowHeight = _props8.rowHeight,
          headerHeight = _props8.headerHeight;

      var rowProps = {
        rowRef: this.getTableHeaderRef,
        width: rowWidth,
        height: rowHeight,
        headerHeight: headerHeight,
        isHeaderRow: true,
        top: 0
      };

      return React.createElement(
        'div',
        {
          ref: function ref(_ref2) {
            _this8.headerWrapper = _ref2;
          },
          className: this.addPrefix('header-row-wrapper')
        },
        this.renderRow(rowProps, headerCells)
      );
    }
  }, {
    key: 'renderTableBody',
    value: function renderTableBody(bodyCells, rowWidth) {
      var _this9 = this;

      var _props9 = this.props,
          headerHeight = _props9.headerHeight,
          rowHeight = _props9.rowHeight,
          rowExpandedHeight = _props9.rowExpandedHeight,
          height = _props9.height,
          data = _props9.data,
          isTree = _props9.isTree,
          setRowHeight = _props9.setRowHeight;
      var tableRowsMaxHeight = this.state.tableRowsMaxHeight;

      var bodyStyles = {
        top: isTree ? 0 : headerHeight || rowHeight,
        height: height - (headerHeight || rowHeight)
      };

      var top = 0; // Row position
      var rows = null;
      if (data && data.length > 0) {
        rows = data.map(function (rowData, index) {
          var maxHeight = tableRowsMaxHeight[index];
          var nextRowHeight = maxHeight ? maxHeight + CELL_PADDING_HEIGHT : rowHeight;
          var shouldRenderExpandedRow = _this9.shouldRenderExpandedRow(rowData);

          if (shouldRenderExpandedRow) {
            nextRowHeight += rowExpandedHeight;
          }

          /**
           * 自定义行高
           */
          if (setRowHeight) {
            nextRowHeight = setRowHeight(rowData) || rowHeight;
          }

          var row = _this9.renderRowData(bodyCells, rowData, {
            index: index,
            top: top,
            rowWidth: rowWidth,
            layer: 0,
            rowHeight: nextRowHeight
          }, shouldRenderExpandedRow);

          !isTree && (top += nextRowHeight);
          return row;
        });
      }

      var wheelStyles = {
        position: 'absolute'
      };

      return React.createElement(
        'div',
        {
          ref: function ref(_ref4) {
            _this9.tableBody = _ref4;
          },
          className: this.addPrefix('body-row-wrapper'),
          style: bodyStyles,
          onTouchStart: this.handleTouchStart,
          onTouchMove: this.handleTouchMove,
          onWheel: this.wheelHandler.onWheel
        },
        React.createElement(
          'div',
          {
            style: wheelStyles,
            className: this.addPrefix('body-wheel-area'),
            ref: function ref(_ref3) {
              _this9.wheelWrapper = _ref3;
            }
          },
          rows
        ),
        this.renderInfo(rows === null),
        this.renderScrollbar(),
        this.renderLoading()
      );
    }
  }, {
    key: 'renderInfo',
    value: function renderInfo(shouldShow) {
      if (!shouldShow) {
        return null;
      }

      var locale = this.props.locale;

      return React.createElement(
        'div',
        { className: this.addPrefix('body-info') },
        locale.emptyMessage
      );
    }
  }, {
    key: 'renderScrollbar',
    value: function renderScrollbar() {
      var _this10 = this;

      var _props10 = this.props,
          disabledScroll = _props10.disabledScroll,
          headerHeight = _props10.headerHeight,
          rowHeight = _props10.rowHeight,
          height = _props10.height,
          loading = _props10.loading;
      var _state = this.state,
          contentWidth = _state.contentWidth,
          contentHeight = _state.contentHeight;


      if (disabledScroll || loading) {
        return null;
      }

      return React.createElement(
        'div',
        null,
        React.createElement(_Scrollbar2.default, {
          length: this.state.width,
          onScroll: this.handleScrollX,
          scrollLength: contentWidth,
          ref: function ref(_ref5) {
            _this10.scrollbarX = _ref5;
          }
        }),
        React.createElement(_Scrollbar2.default, {
          vertical: true,
          length: height - (headerHeight || rowHeight),
          scrollLength: contentHeight,
          onScroll: this.handleScrollY,
          ref: function ref(_ref6) {
            _this10.scrollbarY = _ref6;
          }
        })
      );
    }

    /**
     *  show loading
     */

  }, {
    key: 'renderLoading',
    value: function renderLoading() {
      var _props11 = this.props,
          loading = _props11.loading,
          locale = _props11.locale;


      if (!loading) {
        return null;
      }

      return React.createElement(
        'div',
        { className: this.addPrefix('loader-wrapper') },
        React.createElement(
          'div',
          { className: this.addPrefix('loader') },
          React.createElement('i', { className: this.addPrefix('loader-icon') }),
          React.createElement(
            'span',
            { className: this.addPrefix('loader-text') },
            locale.loading
          )
        )
      );
    }
  }, {
    key: 'render',
    value: function render() {
      var _classNames2,
          _this11 = this;

      var _props12 = this.props,
          children = _props12.children,
          className = _props12.className,
          _props12$width = _props12.width,
          width = _props12$width === undefined ? 0 : _props12$width,
          height = _props12.height,
          style = _props12.style,
          rowHeight = _props12.rowHeight,
          isTree = _props12.isTree,
          hover = _props12.hover,
          bordered = _props12.bordered,
          wordWrap = _props12.wordWrap,
          classPrefix = _props12.classPrefix,
          rest = _objectWithoutProperties(_props12, ['children', 'className', 'width', 'height', 'style', 'rowHeight', 'isTree', 'hover', 'bordered', 'wordWrap', 'classPrefix']);

      var isColumnResizing = this.state.isColumnResizing;

      var _getCells = this.getCells(),
          headerCells = _getCells.headerCells,
          bodyCells = _getCells.bodyCells,
          allColumnsWidth = _getCells.allColumnsWidth;

      var rowWidth = allColumnsWidth > width ? allColumnsWidth : width;
      var clesses = (0, _classnames2.default)(classPrefix, className, (_classNames2 = {}, _defineProperty(_classNames2, this.addPrefix('word-wrap'), wordWrap), _defineProperty(_classNames2, this.addPrefix('treetable'), isTree), _defineProperty(_classNames2, this.addPrefix('bordered'), bordered), _defineProperty(_classNames2, this.addPrefix('column-resizing'), isColumnResizing), _defineProperty(_classNames2, this.addPrefix('hover'), hover), _classNames2));

      var styles = _extends({
        width: width || 'auto',
        height: height
      }, style);

      var unhandled = (0, _utils.getUnhandledProps)(Table, rest);

      return React.createElement(
        'div',
        _extends({}, unhandled, {
          className: clesses,
          style: styles,
          ref: function ref(_ref7) {
            _this11.table = _ref7;
          }
        }),
        this.renderTableHeader(headerCells, rowWidth),
        children && this.renderTableBody(bodyCells, rowWidth),
        this.renderMouseArea()
      );
    }
  }]);

  return Table;
}(React.Component);

Table.defaultProps = {
  classPrefix: (0, _utils.defaultClassPrefix)('table'),
  data: [],
  height: 200,
  rowHeight: 46,
  headerHeight: 40,
  rowExpandedHeight: 100,
  sortType: 'asc',
  hover: true,
  rowKey: 'key',
  locale: {
    emptyMessage: 'No data found',
    loading: 'Loading...'
  }
};
Table.handledProps = ['bordered', 'children', 'className', 'classPrefix', 'data', 'defaultExpandAllRows', 'defaultExpandedRowKeys', 'disabledScroll', 'expandedRowKeys', 'headerHeight', 'height', 'hover', 'isTree', 'loading', 'locale', 'onExpandChange', 'onRowClick', 'onScroll', 'onSortColumn', 'onTouchMove', 'onTouchStart', 'renderRowExpanded', 'renderTreeToggle', 'rowExpandedHeight', 'rowHeight', 'rowKey', 'setRowHeight', 'sortColumn', 'sortType', 'style', 'width', 'wordWrap'];
Table.propTypes = {
  width: _propTypes2.default.number,
  data: _propTypes2.default.arrayOf(_propTypes2.default.object).isRequired,
  height: _propTypes2.default.number.isRequired,
  rowHeight: _propTypes2.default.number.isRequired,
  setRowHeight: _propTypes2.default.func,
  rowKey: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number]).isRequired,
  headerHeight: _propTypes2.default.number.isRequired,
  isTree: _propTypes2.default.bool,
  defaultExpandAllRows: _propTypes2.default.bool,
  defaultExpandedRowKeys: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number])),
  expandedRowKeys: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number])),
  renderTreeToggle: _propTypes2.default.func,
  renderRowExpanded: _propTypes2.default.func,
  rowExpandedHeight: _propTypes2.default.number,
  locale: _propTypes2.default.object.isRequired,
  style: _propTypes2.default.object,
  sortColumn: _propTypes2.default.string,
  sortType: _propTypes2.default.oneOf(['desc', 'asc']).isRequired,
  disabledScroll: _propTypes2.default.bool,
  hover: _propTypes2.default.bool.isRequired,
  loading: _propTypes2.default.bool,
  className: _propTypes2.default.string,
  classPrefix: _propTypes2.default.string,
  children: function children() {
    return (typeof (React.ChildrenArray == null ? {} : React.ChildrenArray) === 'function' ? _propTypes2.default.instanceOf(React.ChildrenArray == null ? {} : React.ChildrenArray) : _propTypes2.default.any).apply(this, arguments);
  },
  bordered: _propTypes2.default.bool,
  wordWrap: _propTypes2.default.bool,
  onRowClick: _propTypes2.default.func,
  onScroll: _propTypes2.default.func,
  onSortColumn: _propTypes2.default.func,
  onExpandChange: _propTypes2.default.func,
  onTouchStart: _propTypes2.default.func,
  // for tests
  onTouchMove: _propTypes2.default.func
};

var _initialiseProps = function _initialiseProps() {
  var _this12 = this;

  this.getTableHeaderRef = function (ref) {
    _this12.tableHeader = ref;
  };

  this.handleColumnResizeEnd = function (columnWidth, cursorDelta, dataKey, index) {
    _this12.setState(_defineProperty({
      isColumnResizing: false
    }, dataKey + '_' + index + '_width', columnWidth));

    (0, _domLib.addStyle)(_this12.mouseArea, {
      display: 'none'
    });
  };

  this.handleColumnResizeStart = function (width, left, fixed) {
    _this12.setState({
      isColumnResizing: true
    });
    var mouseAreaLeft = width + left;
    var x = fixed ? mouseAreaLeft : mouseAreaLeft + (_this12.scrollX || 0);
    var styles = { display: 'block' };
    (0, _domLib.translateDOMPositionXY)(styles, x, 0);
    (0, _domLib.addStyle)(_this12.mouseArea, styles);
  };

  this.handleColumnResizeMove = function (width, left, fixed) {
    var mouseAreaLeft = width + left;
    var x = fixed ? mouseAreaLeft : mouseAreaLeft + (_this12.scrollX || 0);
    var styles = {};
    (0, _domLib.translateDOMPositionXY)(styles, x, 0);
    (0, _domLib.addStyle)(_this12.mouseArea, styles);
  };

  this.handleTreeToggle = function (rowKey, rowIndex, rowData) {
    var onExpandChange = _this12.props.onExpandChange;
    var expandedRowKeys = _this12.state.expandedRowKeys;


    var open = false;
    var nextExpandedRowKeys = [];
    expandedRowKeys.forEach(function (key) {
      if (key === rowKey) {
        open = true;
      } else {
        nextExpandedRowKeys.push(key);
      }
    });

    if (!open) {
      nextExpandedRowKeys.push(rowKey);
    }
    _this12.setState({
      expandedRowKeys: nextExpandedRowKeys
    });
    onExpandChange && onExpandChange(!open, rowData);
  };

  this.handleScrollX = function (delta) {
    _this12.handleWheel(delta, 0);
  };

  this.handleScrollY = function (delta) {
    _this12.handleWheel(0, delta);
  };

  this.handleWheel = function (deltaX, deltaY) {
    var onScroll = _this12.props.onScroll;

    if (!_this12.table) {
      return;
    }
    var nextScrollX = _this12.scrollX - deltaX;
    var nextScrollY = _this12.scrollY - deltaY;

    _this12.scrollY = Math.min(0, nextScrollY < _this12.minScrollY ? _this12.minScrollY : nextScrollY);
    _this12.scrollX = Math.min(0, nextScrollX < _this12.minScrollX ? _this12.minScrollX : nextScrollX);
    _this12.updatePosition();

    onScroll && onScroll(_this12.scrollX, _this12.scrollY);
  };

  this.handleTouchStart = function (event) {
    var onTouchStart = _this12.props.onTouchStart;

    var _ref8 = event.touches ? event.touches[0] : {},
        pageX = _ref8.pageX,
        pageY = _ref8.pageY;

    _this12.touchX = pageX;
    _this12.touchY = pageY;
    onTouchStart && onTouchStart(event);
  };

  this.handleTouchMove = function (event) {
    event.stopPropagation();
    event.preventDefault();

    var onTouchMove = _this12.props.onTouchMove;

    var _ref9 = event.touches ? event.touches[0] : {},
        nextPageX = _ref9.pageX,
        nextPageY = _ref9.pageY;

    var deltaX = _this12.touchX - nextPageX;
    var deltaY = _this12.touchY - nextPageY;
    _this12.handleWheel(deltaX, deltaY);
    _this12.scrollbarX.onWheelScroll(deltaX);
    _this12.scrollbarY.onWheelScroll(deltaY);
    _this12.touchX = nextPageX;
    _this12.touchY = nextPageY;

    onTouchMove && onTouchMove(event);
  };

  this.shouldHandleWheelX = function (delta) {
    var _props13 = _this12.props,
        disabledScroll = _props13.disabledScroll,
        loading = _props13.loading;
    var _state2 = _this12.state,
        contentWidth = _state2.contentWidth,
        width = _state2.width;

    if (delta === 0 || disabledScroll || loading) {
      return false;
    }

    if (width && contentWidth <= width) {
      return false;
    }

    return delta >= 0 && _this12.scrollX > _this12.minScrollX || delta < 0 && _this12.scrollX < 0;
  };

  this.shouldHandleWheelY = function (delta) {
    var _props14 = _this12.props,
        disabledScroll = _props14.disabledScroll,
        loading = _props14.loading;

    if (delta === 0 || disabledScroll || loading) {
      return false;
    }
    return delta >= 0 && _this12.scrollY > _this12.minScrollY || delta < 0 && _this12.scrollY < 0;
  };

  this.tableRows = [];
  this.mounted = false;
  this.scrollY = 0;
  this.scrollX = 0;

  this.addPrefix = function (name) {
    return (0, _utils.prefix)(_this12.props.classPrefix)(name);
  };

  this.calculateTableWidth = function () {
    var table = _this12.table;
    if (table) {
      _this12.scrollX = 0;
      _this12.scrollbarX && _this12.scrollbarX.resetScrollBarPosition();
      _this12.setState({
        width: (0, _domLib.getWidth)(table)
      });
    }
  };
};

exports.default = Table;